# Simple text terminal with Raspberry Pico
# For details see:
# www.homofaciens.de/technics-physical-computing-Secure-Communication-Terminal_en.htm

from ili934xnew import ILI9341, color565
from machine import Pin, PWM, SPI
from micropython import const
import os
import glcdfont
import tt14
import tt24
import tt32
import time
import utime

SCR_WIDTH = const(320)
SCR_HEIGHT = const(240)
SCR_ROT = const(0)
CENTER_Y = int(SCR_WIDTH/2)
CENTER_X = int(SCR_HEIGHT/2)

print(os.uname())
TFT_CLK_PIN = const(6)
TFT_MOSI_PIN = const(7)
TFT_MISO_PIN = const(4)

TFT_CS_PIN = const(13)
TFT_RST_PIN = const(14)
TFT_DC_PIN = const(15)

fonts = [glcdfont,tt14,tt24,tt32]
text = 'Hello Raspberry Pi Pico/ili9341'

print(text)
print("fonts available:")
for f in fonts:
    print(f.__name__)

spi = SPI(
    0,
    baudrate=40000000,
    miso=Pin(TFT_MISO_PIN),
    mosi=Pin(TFT_MOSI_PIN),
    sck=Pin(TFT_CLK_PIN))
print(spi)

display = ILI9341(
    spi,
    cs=Pin(TFT_CS_PIN),
    dc=Pin(TFT_DC_PIN),
    rst=Pin(TFT_RST_PIN),
    w=SCR_WIDTH,
    h=SCR_HEIGHT,
    r=SCR_ROT)

display.erase()
display.set_pos(0,0)

disRmax = 260

display.set_font(tt24)
display.set_color(color565(0, 0, 155), color565(150, 150, 150))
display.fill_rectangle(0,disRmax,240,320,color565(150, 150, 150))


textMax = 9
textLines = ["","","","","","","","","",""]


padMatrix = [['1ABC','2DEF','3GHI'],
             ['4JKL','5MNO','6PQR'],
             ['7STU','8VWX','9YZ!'],
             ['*+-,','0 ?.','####']]

columnPins = [0, 1, 2]
rowPins = [3, 4, 5, 8]

irSensor = Pin(17, Pin.IN)

irDiode = Pin(16, Pin.OUT)


cP = [Pin(2, Pin.OUT),Pin(1, Pin.OUT),Pin(0, Pin.OUT)]
rP = [Pin(8, Pin.IN, Pin.PULL_UP),Pin(5, Pin.IN, Pin.PULL_UP),Pin(4, Pin.IN, Pin.PULL_UP),Pin(3, Pin.IN, Pin.PULL_UP)]

for j in range(3):
    cP[j].value(1)

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def receiveMessage():
    beepPause = utime.ticks_ms()

    messageText = ""
    receiveChar = 0
    receiveBit = 0
    readState = 1
    while utime.ticks_diff(utime.ticks_ms(), beepPause) < 1500 and readState == 1:
        while irSensor.value()==0:
            pass
        beepPause = utime.ticks_ms()
        while irSensor.value() == 1 and readState == 1:
            if utime.ticks_diff(utime.ticks_ms(), beepPause) > 1500:
                readState = 0
        if readState == 1 and receiveBit < 8:
            if utime.ticks_diff(utime.ticks_ms(), beepPause) > 30:
                if utime.ticks_diff(utime.ticks_ms(), beepPause) > 60:
                    receiveChar = receiveChar | 1<<receiveBit
                    receiveBit = receiveBit + 1
                else:
                    receiveBit = receiveBit + 1

        if receiveBit == 8:
            messageText = messageText + chr(receiveChar)
            receiveBit = 0
            receiveChar = 0

    for i in range(textMax+1):
        if textLines[i] == "":
            textLines[i] = "R" + messageText
            if i == textMax:
                scrollText()
                textLines[i] = ""
            break
    printScreen()

    printScreen()
    return 0


# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def sendChar(c):
    for i in range(8):
        for n in range(10): # Send 10 ir pulses
            irDiode.value(1)
            time.sleep(0.000035)
            irDiode.value(0)
            time.sleep(0.000035)
        if ord(c) & 1<<i:
            time.sleep(0.1)
        else:
            time.sleep(0.05)

    return 0

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def sendMessage(sendString):
    Dcol = 0
    Drow = 200

    for i in range(50): # Send pulses to initiate communication
        for n in range(10): # Send 10 ir pulses
            irDiode.value(1)
            time.sleep(0.000035)
            irDiode.value(0)
            time.sleep(0.000035)
        time.sleep(0.001)

    display.set_color(color565(255, 0, 0), color565(0, 0, 0))
    for c in sendString[0:len(sendString)]:
        sendChar(c)

    for n in range(10): # terminating pulses
        irDiode.value(1)
        time.sleep(0.000035)
        irDiode.value(0)
        time.sleep(0.000035)

    time.sleep(0.1)
    display.set_color(color565(255, 255, 0), color565(0, 0, 0))
    return 0

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def keypad():
    keyPressed = ""
    charSel = 0
    timeStart = utime.ticks_ms()
    while True:
        for j in range(3):
            cP[j].value(0)
            for i in range(4):
                if rP[i].value() == 0:
                    timeStart = utime.ticks_ms()
                    keyPressed = padMatrix[i][j][charSel:charSel + 1]
                    while rP[i].value() == 0:
                        pass

                    time.sleep(0.01) # surpress contact bounce
                    charSel = charSel + 1
                    if charSel > 3:
                        charSel = 0
            cP[j].value(1)
        if (keyPressed != "") and (utime.ticks_diff(utime.ticks_ms(), timeStart) > 500):
            return keyPressed
        if irSensor.value() == 0:
            receiveMessage()
    return False

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def scrollText():
    for i in range(textMax):
        textLines[i] = textLines[i+1]
    textLines[textMax] = ""

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def printScreen():
    display.set_color(color565(255, 255, 0), color565(0, 0, 0))
    display.erase()
    display.set_pos(0,0)
    for i in range(textMax+1):
        if textLines[i][0:1] == "R":
            display.set_color(color565(0, 255, 0), color565(0, 0, 0))
        else:
            display.set_color(color565(255, 255, 0), color565(0, 0, 0))
        display.write(textLines[i][1:])
        display.write("\n")
    display.set_color(color565(0, 0, 155), color565(150, 150, 150))
    display.fill_rectangle(0,disRmax,240,320,color565(150, 150, 150))


sendString = ""
printScreen()
# ###############################################################
while True:
  char = keypad()
  if char == "*": # delete character
      if len(sendString) > 0:
          sendString = sendString[0:len(sendString)-1]
      display.set_pos(0,disRmax)
      display.write(sendString + "   ")

  elif char == "#":
      sendMessage(sendString)

      for i in range(textMax+1):
          if textLines[i] == "":
              textLines[i] = "S" + sendString
              if i == textMax:
                  scrollText()
                  textLines[i] = ""
              break
      printScreen()
      sendString = ""
      display.fill_rectangle(0,disRmax,240,320,color565(150, 150, 150))

  else:
      sendString = sendString + char
      display.set_pos(0,disRmax)
      display.write(sendString + "   ")


print("...should never be here.")
